#include <stdio.h>
#include <stdlib.h>
#include <pcap.h>

#define PACKETCOUNT 300000

u_char * fullpacket;
int pktlen;

int main(int argc, char * argv[]) {
	char * ebuf;
	char * fn;

	pcap_t * ifile;
	struct pcap_pkthdr *  pktHeader = NULL;

	unsigned long counter;

	if (argc != 2) {
		fprintf (stderr, "Usage: translator inputfile\n");
		fprintf (stderr, "Defaulting to filename webtrace\n");
		fn = "webtrace";
		exit(1);
	} else {
		fn = argv[1];
	}


	ebuf = malloc( PCAP_ERRBUF_SIZE );
	if (ebuf == NULL) {
		fprintf(stderr, "No mem\n");
		exit(1);
	}

	ifile = pcap_open_offline(fn, ebuf);
	if (ifile == NULL) {
		fprintf(stderr, "Error opening file\n");
		exit(1);
	}

	pktHeader = malloc (sizeof(struct pcap_pkthdr));
	if (pktHeader == NULL) {
		fprintf(stderr, "No mem\n");
		exit(1);
	}
	
	counter = 1;
	fullpacket = (u_char*)pcap_next(ifile, pktHeader);
	/* file header */
	printf("# IXP1200 Developer Workbench Data Stream File\n");
	printf("# Format Version 2.00\n");
	printf("#********  Do not edit this file ***********\n");
	printf("\n");
	printf("# Begin Data Stream ixp_webtrace\n");
	printf("STREAM_TYPE = Ethernet TCP/IP\n");
	while (fullpacket != NULL) {
		int i;
		int totLen = pktHeader->caplen;
		int ehl = 14;
		int iphl = (fullpacket[14] & 0xF) << 2; //ip header length in bytes
		int tcphl = (totLen>14+iphl+12)?((fullpacket[14+iphl+12] & 0xF0) >> 2) : 0;
		if (tcphl==0) {
			fprintf(stderr, "Encountered non tcp packet...ignoring\n");
		} else {


			/* ethernet frame */
			printf("# Begin Ethernet Frame\n");

			/* ethernet header */
			printf("# Begin Ethernet Header\n");
			printf("FIXED_DATA = ");
			for (i=0; i<ehl; i++) {
				printf("%02x",fullpacket[i]);
			}
			printf("\n");
			printf("# End Ethernet Header\n");

			/* begin ip packet */
			printf("# Begin Ip Packet\n");

			/* ip header */
			printf("# Begin Ip Header\n");
			printf("COMPUTE_CHECKSUM = TRUE\n");
			printf("COMPUTE_PACKET_LENGTH = TRUE\n");
			printf("FIXED_DATA = ");
			for (i=ehl; i<ehl+iphl; i++) {
				printf("%02x",fullpacket[i]);
			}
			printf("\n");
			printf("# End Ip Header\n");

			/* tcp segment */
			printf("# Begin TCP Segment\n");

			/* tcp header */
			printf("# Begin TCP Header\n");
			printf("COMPUTE_CHECKSUM = TRUE\n");
			printf("FIXED_DATA = ");
			for (i=ehl+iphl; i<ehl+iphl+tcphl; i++) {
				printf("%02x",fullpacket[i]);
			}
			printf("\n");
			printf("# End TCP Header\n");

			/* payload */
			printf("# Begin Data Payload\n");
			printf("USE_FILL_PATTERN = FALSE\n");
			printf("DATA_PAYLOAD_SIZE = %d\n", (totLen-ehl-iphl-tcphl));
			if ( (totLen-ehl-iphl-tcphl) ) {
				int c=0;
				printf("DATA_PAYLOAD = ");
				for (i=ehl+iphl+tcphl; i<totLen; i++) {
					printf("%02x",fullpacket[i]);
					c++;
					if (c==1000) {
						printf("\nDATA_PAYLOAD = ");
					}
				}
				printf("\n");
			} else {
				printf("DATA_PAYLOAD = 0\n");
			}
			printf("# End Data Payload\n");
			printf("# End TCP Segment\n");
			printf("# End Ip Packet\n");

			/* ethernet trailer */
			printf("# Begin Ethernet Trailer\n");
			printf("FIXED_DATA = ");
			for (i=0; i<4; i++) {
				printf("%02x",0);
			}
			printf("\n");
			printf("# End Ethernet Trailer\n");
			printf("# End Ethernet Frame\n");
			if (PACKETCOUNT && counter == PACKETCOUNT){ break; }
			counter++;
		}
		fullpacket = (u_char*)pcap_next(ifile, pktHeader);
	} 
	printf("# End Data Stream\n");
	

	pcap_close(ifile);

	return 0;
}

